#========================================================================================
# package      DE_EPAGES::Ipayment::t::CGI_CreditCard_Amex_Negative_si
# state        public
#----------------------------------------------------------------------------------------
# description  System Integration Test for Credit Card (AmexCard) Transactions
#========================================================================================
package DE_EPAGES::Ipayment::t::CGI_CreditCard_Amex_Negative_si;
use base DE_EPAGES::Shop::API::TestImportXML;

use strict;

use Readonly;
use URI;
use URI::QueryParam;

use DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth;
use DE_EPAGES::Ipayment::API::Order::Payment;
use DE_EPAGES::Ipayment::API::Session;
use DE_EPAGES::Ipayment::API::SOAP::Request::HashBuilder;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );

use Test::More;
use Test::MockObject::Extends;

Readonly my $SHOPXML_CREDITCARD => 'ShopXMLs/ShopWithIpaymentCreditCard_Amex.xml';

#----------------------------------------------------------------------------------------
# Wrap for DE_EPAGES::Shop::API::TestImportXML
# You need to override this methods in other to use
#----------------------------------------------------------------------------------------
sub shopAlias {
    return 'TestShopIpayment';
}
sub importXMLPath {
    return 'ShopXMLs/ShopWithIpaymentCreditCard.xml';
}

#----------------------------------------------------------------------------------------

sub test {
    my $self = shift;

    my $Order;
    my $SimulateHiddenTriggerCall;
    Readonly my $ORDERPATH => $self->shopPath().'/Customers/TestCustomer/Orders/1234-5678';

    # Preauth With Invalid CreditCard Data
    $Order = LoadObjectByPath($ORDERPATH);
    _createSession($Order);
    _preauthTransactionWithInvalidCreditCardData($Order);


    # Preauth without calling hidden trigger and capture
    $SimulateHiddenTriggerCall = 0;
    $Order = LoadObjectByPath($ORDERPATH);
    _createSession($Order);
    _preauthTransaction($Order, $SimulateHiddenTriggerCall);
    _captureTransactionWithoutHiddenTriggerBefore($Order);

    # Preauth without calling hidden trigger and cancel
    $SimulateHiddenTriggerCall = 0;
    $Order = LoadObjectByPath($ORDERPATH);
    _createSession($Order);
    _preauthTransaction($Order, $SimulateHiddenTriggerCall);
    _cancelTransactionWithoutHiddenTriggerBefore($Order);


    # Preauth and capture with invalid transaction number
    $SimulateHiddenTriggerCall = 1;
    $Order = LoadObjectByPath($ORDERPATH);
    _createSession($Order);
    _preauthTransaction($Order, $SimulateHiddenTriggerCall);
    _captureTransactionWithInvalidTrxNumber($Order);

    # Preauth and cancel with invalid transaction number
    $SimulateHiddenTriggerCall = 1;
    $Order = LoadObjectByPath($ORDERPATH);
    _createSession($Order);
    _preauthTransaction($Order, $SimulateHiddenTriggerCall);
    _cancelTransactionWithInvalidTrxNumber($Order);

    done_testing();
}
#----------------------------------------------------------------------------------------

sub _preauthTransaction {
    my $SubTestName = (caller(0))[3];
    my ($Order, $SimulateHiddenTriggerCall) = @_;

    my $AuthOrPreauth = DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth->new();
    my $ReturnParams = $AuthOrPreauth->getRequestParameters($Order);
    my $hRequestParams = $ReturnParams->{'HiddenInputs'};
    my $PaymentUrl = $ReturnParams->{'PaymentUrl'};

    $hRequestParams->{'silent'} = '1';
    $hRequestParams->{'cc_number'} = '378282246310005';
    $hRequestParams->{'cc_expdate_month'} = '08';
    $hRequestParams->{'cc_expdate_year'} = '2030';
    $hRequestParams->{'cc_startdate_month'} = '08';
    $hRequestParams->{'cc_startdate_year'} = '2010';

    my $UserAgent = LWP::UserAgent->new();
    my $Response = $UserAgent->post( $PaymentUrl, $hRequestParams );

    my $ReturnUrl = $Response->{'_content'};
    my $RedirectURI = URI->new($ReturnUrl);
    my $ResponseParameters = $RedirectURI->query_form_hash();

    is($ResponseParameters->{'trx_typ'}, 'preauth', "$SubTestName - test if transaction was done with preauth");
    is($ResponseParameters->{'trx_paymentmethod'}, 'AmexCard', "$SubTestName - test if paymentmethod was AmexCard");
    is($ResponseParameters->{'cc_typ'}, 'AmexCard', "$SubTestName - test if credit card type was AmexCard");
    is($ResponseParameters->{'ret_status'}, 'SUCCESS">Silent-Redirect</a><br', "$SubTestName - test if return status was success");


    # Set TransID and TransStatus to prepare the order for capture or cancel.
    if($SimulateHiddenTriggerCall) {
        my $LineItemPaymentIpayment = $Order->container()->get('Payment');
        $LineItemPaymentIpayment->set({
            'TransID' => $ResponseParameters->{''},
            'TransStatus' => 'STATUS_ACCEPTED',
        });
    }
}



sub _captureTransactionWithoutHiddenTriggerBefore {
    my $SubTestName = (caller(0))[3];
    my ($Order) = @_;

    my $TransStatus;
    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    is($TransStatus, undef, "$SubTestName - test if transaction status is undef");

    my $Payment = DE_EPAGES::Ipayment::API::Order::Payment->new();
    $Payment->capture($Order);

    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    is($TransStatus, undef, "$SubTestName - test if transaction status is undef");

}



sub _cancelTransactionWithoutHiddenTriggerBefore {
    my $SubTestName = (caller(0))[3];
    my ($Order) = @_;

    my $TransStatus;
    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    is($TransStatus, undef, "$SubTestName - test if transaction status is undef");

    my $Payment = DE_EPAGES::Ipayment::API::Order::Payment->new();
    $Payment->cancel($Order);

    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    is($TransStatus, undef, "$SubTestName - test if transaction status is undef");

}



sub _captureTransactionWithInvalidTrxNumber {
    my $SubTestName = (caller(0))[3];
    my ($Order) = @_;

    my $TransStatus;
    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    is($TransStatus, 'STATUS_ACCEPTED', "$SubTestName - test if transaction status is accepted");

    my $Payment = DE_EPAGES::Ipayment::API::Order::Payment->new();
    $Payment->capture($Order);

    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    is($TransStatus, 'STATUS_FAILED', "$SubTestName - test if transaction status is failed");

}



sub _cancelTransactionWithInvalidTrxNumber {
    my $SubTestName = (caller(0))[3];
    my ($Order) = @_;

    my $TransStatus;
    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    is($TransStatus, 'STATUS_ACCEPTED', "$SubTestName - test if transaction status is accepted");

    my $Payment = DE_EPAGES::Ipayment::API::Order::Payment->new();
    $Payment->cancel($Order);

    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    is($TransStatus, 'STATUS_FAILED', "$SubTestName - test if transaction status is failed");

}



sub _createSession {
    my $SubTestName = (caller(0))[3];
    my ($Order) = @_;

    #prepare
    my $Session = DE_EPAGES::Ipayment::API::Session->new();
    $Session = Test::MockObject::Extends->new( $Session );
    DisableDoubleTransactionCheck($Session);

    #request
    my $Status = $Session->createSessionAtIpaymentAndStoreIdInDatabase($Order);

    ok($Status, "$SubTestName - tests if the session has be created");
    my $SessionID = $Order->container()->get('Payment')->get('SessionID');
    ok(isValid($SessionID), "$SubTestName - tests if the session is stored in the database");
}



sub DisableDoubleTransactionCheck {
    my ($Session ) = @_;

    $Session->mock('_createHashbuilder', sub {
        my $self = shift;
        my ($DatabaseBasketOrOrder) = @_;
        my $HashBuilder = DE_EPAGES::Ipayment::API::SOAP::Request::HashBuilder->new($DatabaseBasketOrOrder);
        my $MockedHashBuilder = Test::MockObject::Extends->new( $HashBuilder );

        $MockedHashBuilder->mock( '_assembleOptions', sub {
            my $self = shift;
            return {
                    'fromIp' => '',
                    'advancedStrictIdCheck' => '0',
                    'checkFraudattack' => '',
                    'checkDoubleTrx' => '0',
                    'errorLang' => '',
                    'trxLongsafe' => '',
                    'otherOptions' => {
                        'option' => {
                            'key' => '',
                            'value' => ''
                        }
                    },
                    'browserData' => {
                        'browserUserAgent' => '',
                        'browserAcceptHeaders' => '',
                    },
                    'clientData' => {
                        'clientName' => 'ePages',
                        'clientVersion' => '',
                    }
                };
        } );
        return $MockedHashBuilder;
    });
    return undef;
}



sub _preauthTransactionWithInvalidCreditCardData {
    my $SubTestName = (caller(0))[3];
    my ($Order, $SimulateHiddenTriggerCall) = @_;

    my $AuthOrPreauth = DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth->new();
    my $ReturnParams = $AuthOrPreauth->getRequestParameters($Order);
    my $hRequestParams = $ReturnParams->{'HiddenInputs'};
    my $PaymentUrl = $ReturnParams->{'PaymentUrl'};

    $hRequestParams->{'silent'} = '1';
    $hRequestParams->{'cc_number'} = '';
    $hRequestParams->{'cc_expdate_month'} = '';
    $hRequestParams->{'cc_expdate_year'} = '';
    $hRequestParams->{'cc_startdate_month'} = '';
    $hRequestParams->{'cc_startdate_year'} = '';

    my $UserAgent = LWP::UserAgent->new();
    my $Response = $UserAgent->post( $PaymentUrl, $hRequestParams );

    my $ReturnUrl = $Response->{'_content'};
    my $RedirectURI = URI->new($ReturnUrl);
    my $ResponseParameters = $RedirectURI->query_form_hash();

    is($ResponseParameters->{'trx_typ'}, 'preauth', "$SubTestName - test if transaction was done with preauth");
    is($ResponseParameters->{'ret_status'}, 'ERROR">Silent-Redirect</a><br', "$SubTestName - test if return status was success");

}


sub init {
    my $self = shift;
    return 1 if $ENV{'SKIP_CLEANUP'};
    $self->SUPER::init();
    ImportObjects( $SHOPXML_CREDITCARD, $self->shopPath().'/PaymentMethods');
    return 1;
}


__PACKAGE__->runScript();

1;