#========================================================================================
# package      DE_EPAGES::Ipayment::t::CGI_CreditCard_Master_Positive_si
# state        public
#----------------------------------------------------------------------------------------
# description  System Integration Test for Credit Card (MasterCard) Transactions
#========================================================================================
package DE_EPAGES::Ipayment::t::CGI_CreditCard_Master_Positive_si;
use base DE_EPAGES::Shop::API::TestImportXML;

use strict;

use Readonly;
use URI;
use URI::QueryParam;

use DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth;
use DE_EPAGES::Ipayment::API::Order::Payment;
use DE_EPAGES::Ipayment::API::Session;
use DE_EPAGES::Ipayment::API::SOAP::Request::HashBuilder;
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime isValid);

use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );

use Test::More;
use Test::MockObject::Extends;

Readonly my $SHOPXML_CREDITCARD => 'ShopXMLs/ShopWithIpaymentCreditCard_Master.xml';

#----------------------------------------------------------------------------------------
# Wrap for DE_EPAGES::Shop::API::TestImportXML
# You need to override this methods in other to use
#----------------------------------------------------------------------------------------
sub shopAlias {
    return 'TestShopIpayment';
}
sub importXMLPath {
    return 'ShopXMLs/ShopWithIpaymentCreditCard.xml';
}

#----------------------------------------------------------------------------------------

sub test {
    my $self = shift;

    Readonly my $ORDERPATH => $self->shopPath().'/Customers/TestCustomer/Orders/1234-5678';
    my $Order = LoadObjectByPath($ORDERPATH);

    $self->PreauthAndCapture($Order);
    $self->PreauthAndCapture_with_reauth($Order);
    $self->PreauthAndCancel($Order);

    done_testing();
}

#----------------------------------------------------------------------------------------
sub PreauthAndCapture {
    my $self = shift;
    my ($Order) = @_;
    _createSession($Order);
    my $TimePeriodSincePreAuthInDays = 2;
    _preauthTransaction( $Order, $TimePeriodSincePreAuthInDays );
    _captureTransaction($Order);
    return;
}

#----------------------------------------------------------------------------------------
sub PreauthAndCapture_with_reauth {
    my $self = shift;
    my ($Order) = @_;
    _createSession($Order);
    #To proof if this action really uses the re_auth or capture you have to switch on the communication log and look out for 'trx_typ'
    my $TimePeriodSincePreAuthInDays = 8;
    _preauthTransaction( $Order, $TimePeriodSincePreAuthInDays );
    _captureTransaction($Order);
    return;
}

#----------------------------------------------------------------------------------------
sub PreauthAndCancel {
    my $self = shift;
    my ($Order) = @_;
    _createSession( $Order );
    my $TimePeriodSincePreAuthInDays = 2;
    _preauthTransaction( $Order, $TimePeriodSincePreAuthInDays );
    _cancelTransaction( $Order );
    return;
}

#----------------------------------------------------------------------------------------
sub _preauthTransaction {
    my $SubTestName = (caller(0))[3];
    my ($Order , $TimePeriodSincePreAuth) = @_;
    my $AuthOrPreauth = DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth->new();
    my $ReturnParams = $AuthOrPreauth->getRequestParameters($Order);

    my $hRequestParams = $ReturnParams->{'HiddenInputs'};
    my $PaymentUrl = $ReturnParams->{'PaymentUrl'};

    $hRequestParams->{'silent'} = '1';
    $hRequestParams->{'cc_number'} = '5555555555554444';
    $hRequestParams->{'cc_expdate_month'} = '08';
    $hRequestParams->{'cc_expdate_year'} = '2030';
    $hRequestParams->{'cc_startdate_month'} = '08';
    $hRequestParams->{'cc_startdate_year'} = '2010';

    my $UserAgent = LWP::UserAgent->new();
    my $Response = $UserAgent->post( $PaymentUrl, $hRequestParams );

    my $ReturnUrl = $Response->{'_content'};
    my $RedirectURI = URI->new($ReturnUrl);
    my $ResponseParameters = $RedirectURI->query_form_hash();

    is($ResponseParameters->{'trx_typ'}, 'preauth', "$SubTestName - test if transaction was done with preauth");
    is($ResponseParameters->{'trx_paymentmethod'}, 'MasterCard', "$SubTestName - test if paymentmethod was MasterCard");
    is($ResponseParameters->{'cc_typ'}, 'MasterCard', "$SubTestName - test if credit card type was MasterCard");
    is($ResponseParameters->{'ret_status'}, 'SUCCESS">Silent-Redirect</a><br', "$SubTestName - test if return status was success");

    # Set TransID and TransStatus to prepare the order for capture or cancel.
    my $LineItemPaymentIpayment = $Order->container()->get('Payment');
    $LineItemPaymentIpayment->set({
        'TransID' => $ResponseParameters->{'ret_trx_number'},
        'TransStatus' => 'STATUS_ACCEPTED',
        'TransTime' => GetCurrentDateTime->subtract( 'days' => $TimePeriodSincePreAuth ),
    });

}



sub _captureTransaction {
    my $SubTestName = (caller(0))[3];
    my ($Order) = @_;

    my $TransStatus;
    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    is($TransStatus, 'STATUS_ACCEPTED', "$SubTestName - test if transaction status is accepted");

    my $Payment = DE_EPAGES::Ipayment::API::Order::Payment->new();
    $Payment->capture($Order);

    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    is($TransStatus, 'STATUS_PAID', "$SubTestName - test if transaction status is paid");

}



sub _cancelTransaction {
    my $SubTestName = (caller(0))[3];
    my ($Order) = @_;

    my $TransStatus;
    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    is($TransStatus, 'STATUS_ACCEPTED', "$SubTestName - test if transaction status is accepted");

    my $Payment = DE_EPAGES::Ipayment::API::Order::Payment->new();
    $Payment->cancel($Order);

    $TransStatus = $Order->container()->get('Payment')->get('TransStatus');
    is($TransStatus, 'STATUS_CANCELLED', "$SubTestName - test if transaction status is canceled");

}



sub _createSession {
    my $SubTestName = (caller(0))[3];
    my ($Order) = @_;

    #prepare
    my $Session = DE_EPAGES::Ipayment::API::Session->new();
    $Session = Test::MockObject::Extends->new( $Session );
    DisableDoubleTransactionCheck($Session);

    #request
    my $Status = $Session->createSessionAtIpaymentAndStoreIdInDatabase($Order);

    ok($Status, "$SubTestName - tests if the session has be created");
    my $SessionID = $Order->container()->get('Payment')->get('SessionID');
    ok(isValid($SessionID), "$SubTestName - tests if the session is stored in the database");
}



sub DisableDoubleTransactionCheck {
    my ($Session ) = @_;

    $Session->mock('_createHashbuilder', sub {
        my $self = shift;
        my ($DatabaseBasketOrOrder) = @_;
        my $HashBuilder = DE_EPAGES::Ipayment::API::SOAP::Request::HashBuilder->new($DatabaseBasketOrOrder);
        my $MockedHashBuilder = Test::MockObject::Extends->new( $HashBuilder );

        $MockedHashBuilder->mock( '_assembleOptions', sub {
            my $self = shift;
            return {
                    'fromIp' => '',
                    'advancedStrictIdCheck' => '0',
                    'checkFraudattack' => '',
                    'checkDoubleTrx' => '0',
                    'errorLang' => '',
                    'trxLongsafe' => '',
                    'otherOptions' => {
                        'option' => {
                            'key' => '',
                            'value' => ''
                        }
                    },
                    'browserData' => {
                        'browserUserAgent' => '',
                        'browserAcceptHeaders' => '',
                    },
                    'clientData' => {
                        'clientName' => 'ePages',
                        'clientVersion' => '',
                    }
                };
        } );
        return $MockedHashBuilder;
    });
    return undef;
}

sub init {
    my $self = shift;
    return 1 if $ENV{'SKIP_CLEANUP'};
    $self->SUPER::init();
    ImportObjects( $SHOPXML_CREDITCARD, $self->shopPath().'/PaymentMethods');
    return 1;
}


__PACKAGE__->runScript();

1;