#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::CGI_Cancel
# state        public
#----------------------------------------------------------------------------------------
# description  Unit Test for Cancel.pm
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::CGI_Cancel;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Ipayment::API::CGI::RequestFactory  qw ( CreateDefaultCancel CreateCancelWithCustomUserAgentAndResponse );
use DE_EPAGES::Ipayment::API::CGI::Cancel;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;
use Test::MockObject;
use Test::MockObject::Extends;
use DE_EPAGES::Ipayment::t::unit::MockTools qw ( GetOrder );
use DE_EPAGES::Ipayment::t::unit::TestTools qw (
    CreateMockedUserAgentForPostRequests
    CreateNegativeXMLHttpResponse
    CreateRedirectHttpResponse
);
use DE_EPAGES::Ipayment::API::Constants qw (%CGI_MESSAGES);
use DE_EPAGES::Ipayment::API::Database::BasketOrOrder;



sub Main {

    Test_PositiveCancel();
    Test_NegativeCancel();
    Test_WithInvalidOrder_NoLineItem();
    Test_WithUnsupportedTransStatus();
    Test_WithTransStatusFAILED();

    done_testing();
}



sub Test_WithInvalidOrder_NoLineItem {
    my $SubTestName = ( caller(0) )[3];

    my $Order  = 'This Is An Invalid Order!';

    my $Cancel = CreateDefaultCancel();
    $Cancel = Test::MockObject::Extends->new($Cancel);
    $Cancel->mock('_LogPayment', sub {});
    $Cancel->setDatabaseBasketOrOrder(_GetMockedDatabaseBasketOrOrder());

    my $CancelStatus = $Cancel->cancelMoney($Order);
    is( $CancelStatus, $CGI_MESSAGES{'NO_IPAYMENT_LINEITEM'},
        "$SubTestName - Test If Order contains no LineItemIpayment" );
}



sub Test_WithUnsupportedTransStatus {
    my $SubTestName = ( caller(0) )[3];

    my $Order = GetOrder();
    my $LineItemContainer = $Order->container();
    my $LineItemConcainterIpayment = $LineItemContainer->payment();
    $LineItemConcainterIpayment->set({'TransStatus' => 'STATUS_TEST'});

    my $Cancel = CreateDefaultCancel();
    $Cancel = Test::MockObject::Extends->new($Cancel);
    $Cancel->mock('_LogPayment', sub {});
    $Cancel->setDatabaseBasketOrOrder(_GetMockedDatabaseBasketOrOrder());

    my $CancelStatus = $Cancel->cancelMoney($Order);
    is( $CancelStatus, $CGI_MESSAGES{'UNSUPPORTED_ACTION'},
        "$SubTestName - Test If the action is unsupported for this kind of payment" );
}



sub Test_WithTransStatusFAILED {
    my $SubTestName = ( caller(0) )[3];

    my $Order = GetOrder();
    my $LineItemContainer = $Order->container();
    my $LineItemConcainterIpayment = $LineItemContainer->payment();
    $LineItemConcainterIpayment->set({'TransStatus' => 'STATUS_FAILED'});

    my $PostResponse = CreateRedirectHttpResponse('http://www.redirect.url?ret_status=SUCCESS');
    my $MockedUserAgent = CreateMockedUserAgentForPostRequests($PostResponse);
    my $MockedResponse = _GetMockedResponse();

    my $Cancel = CreateCancelWithCustomUserAgentAndResponse($MockedUserAgent, $MockedResponse);
    $Cancel = Test::MockObject::Extends->new($Cancel);
    $Cancel->mock('_BuildShopUrl', sub {
        return 'www.test.url';
    });
    $Cancel->mock('_UpdateTransactionStatus', sub {});
    $Cancel->mock('_LogPayment', sub {});
    $Cancel->setDatabaseBasketOrOrder(_GetMockedDatabaseBasketOrOrder());

    my $CancelStatus = $Cancel->cancelMoney($Order);
    is( $CancelStatus, $CGI_MESSAGES{'OK'}, "$SubTestName - Test If a Order which was Failed before is working again" );
}



sub Test_PositiveCancel {
    my $SubTestName = (caller(0))[3];

    my $PostResponse = CreateRedirectHttpResponse('http://www.redirect.url?ret_status=SUCCESS');
    my $MockedUserAgent = CreateMockedUserAgentForPostRequests($PostResponse);
    my $MockedResponse = _GetMockedResponse();

    my $Cancel = CreateCancelWithCustomUserAgentAndResponse($MockedUserAgent, $MockedResponse);
    $Cancel = Test::MockObject::Extends->new($Cancel);
    $Cancel->mock('_BuildShopUrl', sub {
        return 'www.test.url';
    });
    $Cancel->mock('_UpdateTransactionStatus', sub {});
    $Cancel->mock('_LogPayment', sub {});
    $Cancel->setDatabaseBasketOrOrder(_GetMockedDatabaseBasketOrOrder());

    my $Order = GetOrder();

    my $CancelStatus = $Cancel->cancelMoney($Order);
    is($CancelStatus, $CGI_MESSAGES{'OK'},"$SubTestName - Test if CancelStatus is 1");
}



sub Test_NegativeCancel {
    my $SubTestName = (caller(0))[3];

    my $PostResponse = CreateNegativeXMLHttpResponse('');
    my $MockedUserAgent = CreateMockedUserAgentForPostRequests($PostResponse);
    my $MockedResponse = _GetMockedResponse();

    my $Cancel = CreateCancelWithCustomUserAgentAndResponse($MockedUserAgent, $MockedResponse);
    $Cancel = Test::MockObject::Extends->new($Cancel);
    $Cancel->mock('_BuildShopUrl', sub {
        return 'www.test.url';
    });
    $Cancel->mock('_UpdateTransactionStatus', sub {});
    $Cancel->mock('_LogPayment', sub {});
    $Cancel->setDatabaseBasketOrOrder(_GetMockedDatabaseBasketOrOrder());

    my $Order = GetOrder();

    my $CancelStatus = $Cancel->cancelMoney($Order);
    is($CancelStatus, $CGI_MESSAGES{'NO_REDIRECT_URL'},"$SubTestName - Test if CancelStatus is 0");
}



sub _GetMockedResponse {
    my $hMockedAttributes = {};
    my $MockedResponse = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Ipayment::API::CGI::Response', $hMockedAttributes
        );
    $MockedResponse->mock('cancelTransaction', sub {});
    return $MockedResponse;
}



sub _GetMockedDatabaseBasketOrOrder {
    my $DatabaseBasketOrOrder = DE_EPAGES::Ipayment::API::Database::BasketOrOrder->new();
    $DatabaseBasketOrOrder = Test::MockObject::Extends->new($DatabaseBasketOrOrder);
    $DatabaseBasketOrOrder->mock('_BuildShopUrl', sub {});
    return $DatabaseBasketOrOrder;
}



RunScript( Sub => \&Main );