#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::CGI_Capture
# state        public
#----------------------------------------------------------------------------------------
# description  Unit Test for Capture.pm
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::CGI_Capture;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Ipayment::API::CGI::RequestFactory  qw ( CreateDefaultCapture CreateCaptureWithCustomUserAgentAndResponse );
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;
use Test::MockObject::Extends;
use DE_EPAGES::Ipayment::t::unit::MockTools qw ( GetOrder );
use DE_EPAGES::Ipayment::t::unit::TestTools qw (
    CreateMockedUserAgentForPostRequests
    CreateNegativeXMLHttpResponse
    CreateRedirectHttpResponse
);
use DE_EPAGES::Ipayment::API::Constants qw (%CGI_MESSAGES);
use DE_EPAGES::Ipayment::API::Database::BasketOrOrder;



sub Main {

    Test_PositiveCapture();
    Test_NegativeCapture();
    Test_WithInvalidOrder();
    Test_WithUnsupportedTransStatus();
    Test_WithTransStatusFAILED();

    done_testing();
}



sub Test_WithInvalidOrder {
    my $SubTestName = ( caller(0) )[3];

    my $Order  = 'This Is An Invalid Order!';

    my $Capture = CreateDefaultCapture();
    $Capture = Test::MockObject::Extends->new($Capture);
    $Capture->mock('_LogPayment', sub {});
    $Capture->setDatabaseBasketOrOrder(_GetMockedDatabaseBasketOrOrder());

    my $CaptureStatus = $Capture->captureMoney($Order);
    is( $CaptureStatus, $CGI_MESSAGES{'NO_IPAYMENT_LINEITEM'},
        "$SubTestName - Test If Order contains no LineItemIpayment" );
}



sub Test_WithUnsupportedTransStatus {
    my $SubTestName = ( caller(0) )[3];

    my $Order = GetOrder();
    my $LineItemContainer = $Order->container();
    my $LineItemConcainterIpayment = $LineItemContainer->payment();
    $LineItemConcainterIpayment->set({'TransStatus' => 'STATUS_TEST'});

    my $Capture = CreateDefaultCapture();
    $Capture = Test::MockObject::Extends->new($Capture);
    $Capture->mock('_LogPayment', sub {});
    $Capture->setDatabaseBasketOrOrder(_GetMockedDatabaseBasketOrOrder());

    my $CaptureStatus = $Capture->captureMoney($Order);
    is( $CaptureStatus, $CGI_MESSAGES{'UNSUPPORTED_ACTION'},
        "$SubTestName - Test If Order contains no LineItemIpayment" );
}


sub Test_WithTransStatusFAILED {
    my $SubTestName = ( caller(0) )[3];

    my $Order = GetOrder();
    my $LineItemContainer = $Order->container();
    my $LineItemConcainterIpayment = $LineItemContainer->payment();
    $LineItemConcainterIpayment->set({'TransStatus' => 'STATUS_FAILED'});

    my $PostResponse = CreateRedirectHttpResponse('http://www.redirect.url?ret_status=SUCCESS');
    my $MockedUserAgent = CreateMockedUserAgentForPostRequests($PostResponse);
    my $MockedResponse = _GetMockedResponse();

    my $Capture = CreateCaptureWithCustomUserAgentAndResponse($MockedUserAgent, $MockedResponse);
    $Capture = Test::MockObject::Extends->new($Capture);
    $Capture->mock('_BuildShopUrl', sub {
        return 'www.test.url';
    });
    $Capture->mock('_UpdateTransactionStatus', sub {});
    $Capture->mock('_LogPayment', sub {});
    $Capture->setDatabaseBasketOrOrder(_GetMockedDatabaseBasketOrOrder());

    my $CaptureStatus = $Capture->captureMoney($Order);
    is( $CaptureStatus, $CGI_MESSAGES{'OK'}, "$SubTestName - Test If a Order which was Failed before is working again" );
}


sub Test_PositiveCapture {
    my $SubTestName = (caller(0))[3];

    my $PostResponse = CreateRedirectHttpResponse('http://www.redirect.url?ret_status=SUCCESS');
    my $MockedUserAgent = CreateMockedUserAgentForPostRequests($PostResponse);
    my $MockedResponse = _GetMockedResponse();

    my $Capture = CreateCaptureWithCustomUserAgentAndResponse($MockedUserAgent, $MockedResponse);
    $Capture = Test::MockObject::Extends->new($Capture);
    $Capture->mock('_BuildShopUrl', sub {
        return 'www.test.url';
    });
    $Capture->mock('_UpdateTransactionStatus', sub {});
    $Capture->mock('_LogPayment', sub {});
    $Capture->setDatabaseBasketOrOrder(_GetMockedDatabaseBasketOrOrder());

    my $Order = GetOrder();

    my $CaptureStatus = $Capture->captureMoney($Order);
    is($CaptureStatus, $CGI_MESSAGES{'OK'},,"$SubTestName - Test if CaptureStatus is 'OK'");
}



sub Test_NegativeCapture {
    my $SubTestName = (caller(0))[3];

    my $PostResponse = CreateNegativeXMLHttpResponse('');
    my $MockedUserAgent = CreateMockedUserAgentForPostRequests($PostResponse);
    my $MockedResponse = _GetMockedResponse();

    my $Capture = CreateCaptureWithCustomUserAgentAndResponse($MockedUserAgent, $MockedResponse);
    $Capture = Test::MockObject::Extends->new($Capture);
    $Capture->mock('_BuildShopUrl', sub {
        return 'www.test.url';
    });
    $Capture->mock('_UpdateTransactionStatus', sub {});
    $Capture->mock('_LogPayment', sub {});
    $Capture->setDatabaseBasketOrOrder(_GetMockedDatabaseBasketOrOrder());

    my $Order = GetOrder();

    my $CaptureStatus = $Capture->captureMoney($Order);
    is($CaptureStatus, $CGI_MESSAGES{'NO_REDIRECT_URL'},,"$SubTestName - Test if CaptureStatus is error_ResponseContainsNoRedirectUrl");
}



sub _GetMockedResponse {
    my $hMockedAttributes = {};
    my $MockedResponse = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Ipayment::API::CGI::Response', $hMockedAttributes
        );
    $MockedResponse->mock('captureTransaction', sub {});
    return $MockedResponse;
}



sub _GetMockedDatabaseBasketOrOrder {
    my $DatabaseBasketOrOrder = DE_EPAGES::Ipayment::API::Database::BasketOrOrder->new();
    $DatabaseBasketOrOrder = Test::MockObject::Extends->new($DatabaseBasketOrOrder);
    $DatabaseBasketOrOrder->mock('_BuildShopUrl', sub {});
    return $DatabaseBasketOrOrder;
}



RunScript( Sub => \&Main );