#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::WebServiceClient_internal
# state        public
#----------------------------------------------------------------------------------------
# description  Test Ipayment WebService with a real Connection
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::WebServiceClient_internal;

use strict;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Ipayment::API::SOAP::CompiledClient;
use DE_EPAGES::Ipayment::API::SOAP::Request::HashBuilder;
use DE_EPAGES::Ipayment::API::SOAP::Request::Validator;
use DE_EPAGES::Ipayment::API::SOAP::WebServiceClient;
use DE_EPAGES::Ipayment::t::unit::TestTools qw ( TestExistsMethod );
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::Exception;
use Test::MockObject;
use Test::More;

sub TestPlan {
    Test_Positive();
    Test_WrongRequestDataWithCompleteRequestHash();
    Test_WrongRequestDataWithInCompleteRequestHash();
    Test_ErrorWithNativeDieInWebserviceCall();
    Test_ErrorWithSomeErrorInWebserviceCall();
    Test_MissingWebServiceClientObject();
    Test_MissingHashBuilderObject();
    done_testing();
}

sub Test_Positive{
    my $SubTestName = (caller(0))[3];
    my $CompiledClient = _createMockCompiledClient();
    my $WebserviceClient = _createWebServiceClient($CompiledClient);
    my $IsValidData = 1;
    my $Validator = _CreateValidator($IsValidData);
    $WebserviceClient->setValidator($Validator);
    my $MockedHashBuilder = _CreateMockedHashBuilder();

    my $Response = $WebserviceClient->doRequest($MockedHashBuilder);

    is($Response,'a Response',"$SubTestName - testing response of wsdl11 client request");

    my $ExpectedLogEntry_Request = [ 'Ipayment', 'SOAPRequest', _GetTestAccountDataWithoutPasswords(), 'createSession' ];
    is_deeply(
        $WebserviceClient->{'LogPayment_Params'}->[0],
        $ExpectedLogEntry_Request,
        "$SubTestName - check if the logCommunication mock cointains the expected Trace value"
    );

    my $ExpectedLogEntry_Response = [ 'Ipayment', 'SOAPResponse', 'dummy response', 'createSession' ];
    is_deeply(
        $WebserviceClient->{'LogPayment_Params'}->[1],
        $ExpectedLogEntry_Response,
        "$SubTestName - check if the logCommunication mock cointains the expected Trace value"
    );

}

sub Test_WrongRequestDataWithCompleteRequestHash {
    my $SubTestName = (caller(0))[3];
    my $CompiledClient = _createMockCompiledClient();
    my $WebserviceClient = _createWebServiceClient($CompiledClient);
    my $IsValidHash = 0;
    my $Validator = _CreateValidator($IsValidHash);
    $WebserviceClient->setValidator($Validator);
    my $TestRequestData = {
        'accountData' => {
            'trxpassword' => 'DummyPassword',
            'adminactionpassword' => 'DummyAdminPassword',
        },
        'key' => 'value',
    };
    my $HashBuilder = _CreateMockedHashBuilder($TestRequestData);

    my $Response = $WebserviceClient->doRequest($HashBuilder);

    is($Response, undef , "$SubTestName - check if the doRequest return undef if the Hash is not valid");

    my $ExpectedLogData = [[
            'Ipayment',
            'Data invalid, checked before call',
            {
                 'accountData' => {
                    'trxpassword' => '---replaced---',
                    'adminactionpassword' => '---replaced---',
                 },
                 'key' => 'value',
            },
            'createSession'
          ]];
     is_deeply(
        $WebserviceClient->{'LogPayment_Params'},
        $ExpectedLogData,
        "$SubTestName - check if the logCommunication mock cointains the expected RequestData"
    );
}

sub Test_WrongRequestDataWithInCompleteRequestHash {
    my $SubTestName = (caller(0))[3];
    my $CompiledClient = _createMockCompiledClient();
    my $WebserviceClient = _createWebServiceClient($CompiledClient);
    my $IsValidHash = 0;
    my $Validator = _CreateValidator($IsValidHash);
    $WebserviceClient->setValidator($Validator);
    my $TestRequestData = {
        'accountData' => {
        },
        'key' => 'value',
    };
    my $HashBuilder = _CreateMockedHashBuilder($TestRequestData);

    my $Response = $WebserviceClient->doRequest($HashBuilder);

    is($Response, undef , "$SubTestName - check if the doRequest return undef if the Hash is not valid");

    my $ExpectedLogData = [[
            'Ipayment',
            'Data invalid, checked before call',
            {
                 'accountData' => { },
                 'key' => 'value',
            },
            'createSession'
          ]];
     is_deeply(
        $WebserviceClient->{'LogPayment_Params'},
        $ExpectedLogData,
        "$SubTestName - check if the logCommunication mock cointains the expected incomplete RequestData"
    );
}

sub Test_ErrorWithNativeDieInWebserviceCall{
    my $SubTestName = (caller(0))[3];
    my $CompiledClient = _createMockCompiledClientWithError('NativeDie');
    my $WebserviceClient = _createWebServiceClient($CompiledClient);
    my $IsValidHash = 1;
    my $Validator = _CreateValidator($IsValidHash);
    $WebserviceClient->setValidator($Validator);
    my $HashBuilder = _CreateMockedHashBuilder();
    throws_ok { $WebserviceClient->doRequest($HashBuilder) } (
        qr/NativeDie/,
        "$SubTestName - tests if a native die was thrown"
    );
}

sub Test_ErrorWithSomeErrorInWebserviceCall{
    my $SubTestName = (caller(0))[3];
    my $CompiledClient = _createMockCompiledClientWithError('WebServiceError');
    my $WebserviceClient = _createWebServiceClient($CompiledClient);
    my $IsValidHash = 1;
    my $Validator = _CreateValidator($IsValidHash);
    $WebserviceClient->setValidator($Validator);
    my $HashBuilder = _CreateMockedHashBuilder('SomeData');
    my $Response = $WebserviceClient->doRequest($HashBuilder);

    is($Response, undef , "$SubTestName - check if the doRequest return undef if the Hash is not valid");

    my $ExpectedLogEntry_Response = [ 'Ipayment', 'WebServiceError', 'SomeData', 'createSession' ];
    is_deeply(
        $WebserviceClient->{'LogPayment_Params'}->[0],
        $ExpectedLogEntry_Response,
        "$SubTestName - check if the logCommunication mock cointains the expected Trace value"
    );
}
sub Test_MissingWebServiceClientObject {
    my $SubTestName = (caller(0))[3];
    my $UndefCompiledClient = undef;
    throws_ok { DE_EPAGES::Ipayment::API::SOAP::WebServiceClient->new($UndefCompiledClient) } (
        qr/PARAMETER_MISSING/,
        "$SubTestName - Calling the constructor of WebServiceClient is not possible with undefined CompiledClient"
    );
}

sub Test_MissingHashBuilderObject {
    my $SubTestName = (caller(0))[3];
    my $CompiledClient = _createMockCompiledClient();
    my $WebserviceClient = _createWebServiceClient($CompiledClient);
    my $UndefHashBuilder = undef;
    throws_ok { $WebserviceClient->doRequest($UndefHashBuilder) } (
        qr/PARAMETER_MISSING/,
        "$SubTestName - Calling doRequest of WebServiceClient is not possible with undefined HashBuilder"
    );
}
#----------------------------------------------------------------------------------------
sub _CreateValidator {
    my ($IsValid) = @_;
    my $Validator = DE_EPAGES::Ipayment::API::SOAP::Request::Validator->new();
    my $MockedValidator = Test::MockObject::Extends->new( $Validator );
    TestExistsMethod($Validator,'isValidHash');
    $MockedValidator->mock('isValidHash',sub{ return $IsValid; });
    return $MockedValidator;
}
#----------------------------------------------------------------------------------------
sub _CreateMockedHashBuilder{
    my ($ReturnData) = @_;
    my $HashBuilder = DE_EPAGES::Ipayment::API::SOAP::Request::HashBuilder->new();
    my $MockedHashBuilder = Test::MockObject::Extends->new( $HashBuilder );
    $MockedHashBuilder->mock('getRequestData',sub{return $ReturnData;});
    return $MockedHashBuilder;
}

#----------------------------------------------------------------------------------------
sub _CreateTrace {

    my $Trace = Test::MockObject->new();
    my $RequestString = <<"TESTACCOUNTDATA";
<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="https://ipayment.de/service_v3/binding"><SOAP-ENV:Body><ns1:createSession><accountData><accountId>99999</accountId><trxuserId>99999</trxuserId><trxpassword>0</trxpassword><adminactionpassword>5cfgRT34xsdedtFLdfHxj7tfwx24fe</adminactionpassword></accountData><transactionData><trxAmount>7995</trxAmount><trxCurrency>EUR</trxCurrency><invoiceText></invoiceText><trxUserComment></trxUserComment><shopperId>53E9E327-8AF5-A842-F45D-AC150407A43C</shopperId></transactionData><transactionType>preauth</transactionType><paymentType>cc</paymentType><options><fromIp></fromIp><advancedStrictIdCheck>1</advancedStrictIdCheck><checkFraudattack>0</checkFraudattack><checkDoubleTrx>1</checkDoubleTrx><errorLang></errorLang><trxLongsafe>0</trxLongsafe><otherOptions><option><key></key><value></value></option></otherOptions><browserData><browserUserAgent></browserUserAgent><browserAcceptHeaders></browserAcceptHeaders></browserData><clientData><clientName>ePages</clientName><clientVersion></clientVersion></clientData></options><processorUrls><redirectUrl>http://vm-breitkreutz.intern.epages.de/epages/DemoShop.sf/?ChangeAction=Redirect&amp;ObjectID=17713</redirectUrl><silentErrorUrl>http://vm-breitkreutz.intern.epages.de/epages/DemoShop.sf/?ChangeAction=SilentError&amp;ObjectID=17713</silentErrorUrl><hiddenTriggerUrl>http://vm-breitkreutz.intern.epages.de/epages/DemoShop.sf/?ChangeAction=HiddenTrigger&amp;ObjectID=17713</hiddenTriggerUrl></processorUrls></ns1:createSession></SOAP-ENV:Body></SOAP-ENV:Envelope>
TESTACCOUNTDATA
;
    $Trace->mock('request', sub{ return { '_content' => \$RequestString, }; });
    $Trace->mock('response', sub{ return { '_content' => 'dummy response', }; });

    return $Trace;
}
#----------------------------------------------------------------------------------------
sub _GetTestAccountDataWithoutPasswords {
    return <<"TESTACCOUNTDATA";
<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="https://ipayment.de/service_v3/binding"><SOAP-ENV:Body><ns1:createSession><accountData><accountId>99999</accountId><trxuserId>99999</trxuserId><trxpassword>---replaced---</trxpassword><adminactionpassword>---replaced---</adminactionpassword></accountData><transactionData><trxAmount>7995</trxAmount><trxCurrency>EUR</trxCurrency><invoiceText></invoiceText><trxUserComment></trxUserComment><shopperId>53E9E327-8AF5-A842-F45D-AC150407A43C</shopperId></transactionData><transactionType>preauth</transactionType><paymentType>cc</paymentType><options><fromIp></fromIp><advancedStrictIdCheck>1</advancedStrictIdCheck><checkFraudattack>0</checkFraudattack><checkDoubleTrx>1</checkDoubleTrx><errorLang></errorLang><trxLongsafe>0</trxLongsafe><otherOptions><option><key></key><value></value></option></otherOptions><browserData><browserUserAgent></browserUserAgent><browserAcceptHeaders></browserAcceptHeaders></browserData><clientData><clientName>ePages</clientName><clientVersion></clientVersion></clientData></options><processorUrls><redirectUrl>http://vm-breitkreutz.intern.epages.de/epages/DemoShop.sf/?ChangeAction=Redirect&amp;ObjectID=17713</redirectUrl><silentErrorUrl>http://vm-breitkreutz.intern.epages.de/epages/DemoShop.sf/?ChangeAction=SilentError&amp;ObjectID=17713</silentErrorUrl><hiddenTriggerUrl>http://vm-breitkreutz.intern.epages.de/epages/DemoShop.sf/?ChangeAction=HiddenTrigger&amp;ObjectID=17713</hiddenTriggerUrl></processorUrls></ns1:createSession></SOAP-ENV:Body></SOAP-ENV:Envelope>
TESTACCOUNTDATA
;
}
#----------------------------------------------------------------------------------------
sub _createWebServiceClient {
    my ($CompiledClient) = @_;
    my $WebserviceClient = DE_EPAGES::Ipayment::API::SOAP::WebServiceClient->new($CompiledClient);
    my $MockedWebserviceClient = Test::MockObject::Extends->new( $WebserviceClient );
    $MockedWebserviceClient->mock( '_LogPayment', sub {
            my $self = shift;
            push(@{$self->{'LogPayment_Params'}},\@_);
        } );
    return $MockedWebserviceClient;
}

#----------------------------------------------------------------------------------------
sub _createMockCompiledClient {
    my $CompiledClient = DE_EPAGES::Ipayment::API::SOAP::CompiledClient->new();
    my $MockedCompiledClient = Test::MockObject::Extends->new($CompiledClient);

    my $MockedTrace = _CreateTrace();
    TestExistsMethod($CompiledClient,'createClient');
    $MockedCompiledClient->mock('createClient',
        sub { return
                sub{
                    $MockedCompiledClient->{'createClient_Params'} = \@_;#todo check in unit test
                    return ('a Response', $MockedTrace);
                }
        }
    );

    return $MockedCompiledClient;
}
#----------------------------------------------------------------------------------------
sub _createMockCompiledClientWithError {
    my ($ErrorCode) = @_;
    my $CompiledClient = DE_EPAGES::Ipayment::API::SOAP::CompiledClient->new();
    my $MockedCompiledClient = Test::MockObject::Extends->new($CompiledClient);

    TestExistsMethod($CompiledClient,'createClient');
    $MockedCompiledClient->mock('createClient',
        sub { return
                sub{ Error($ErrorCode); }
        }
    );
    return $MockedCompiledClient;
}

RunScript( Sub => \&TestPlan );

1;